import os
from pathlib import Path
from typing import Dict, List, Tuple

import cognite.airworkflow.constants as const
from cognite.airworkflow.model.config import Config
from cognite.airworkflow.model.deployment import Deployment
from cognite.airworkflow.util import file, yaml


def write_dependency_yamls(config: Dict[Path, Config]):
    for model_path, values in config.items():
        if values.dependencies:
            dep_dict = {i.split("==")[0]: i.split("==")[1] for i in values.dependencies}
            yaml.write_yaml(
                model_path / const.FUNCTION_REL_DEPLOYMENT_PATH,
                dep_dict,
                create_dirs=True,
            )


def get_parallel_deployment(model_name: str, project: str, config: Dict[Path, Config] = None) -> List[Deployment]:
    config = config if config else get_master_config()[0]
    for path, info in config.items():
        if path.name == model_name:
            model_version = info.model_version
            return [Deployment(project, path, model_version, info.secret_names)]

    raise ValueError(f"Model/Function named {model_name} is not defined. Please check that it's not ignored.")


def get_function_paths() -> List[Path]:
    ignore_models = file.read_file_to_list(const.IGNORE_MODELS_PATH)
    directory_contents = os.listdir(const.FUNCTIONS_PATH)
    directory_paths = list(
        map(
            lambda x: const.FUNCTIONS_PATH / x,
            filter(lambda x: x not in ignore_models, directory_contents),
        )
    )
    function_paths = [x / const.FUNCTION_REL_CONFIG_PATH for x in directory_paths if os.path.isdir(x)]
    return function_paths


def get_project_to_config_map(master_config: Dict[Path, Config]) -> Dict[str, Dict[Path, Config]]:
    project_to_config: Dict[str, Dict[Path, Config]] = {}

    for path, config in master_config.items():
        for project_name in config.deploy:
            project = project_name
            if project not in project_to_config:
                project_to_config[project] = {}
            project_to_config[project][path] = config

    return project_to_config


def get_master_config() -> Tuple[Dict[Path, Config], Dict[str, Dict[Path, Config]]]:
    function_paths = get_function_paths()
    master_config = yaml.load_and_validate_multiple(function_paths, const.CONFIG_SCHEMA_PATH)
    write_dependency_yamls(master_config)
    project_to_config = get_project_to_config_map(master_config)
    return master_config, project_to_config


if __name__ == "__main__":
    print(get_master_config())



import sys
from pathlib import Path
from typing import Dict

from ruamel.yaml import YAML

from cognite.airworkflow.util import env


def load_yaml(path: Path) -> Dict:
    yaml = YAML(typ="safe").load(path)
    assert isinstance(yaml, dict)
    return yaml


def get_url_dict() -> Dict:
    path_to_dict = env.get_env_value("PWD")
    REPO_BASE_DIR = Path(path_to_dict)
    try:
        repo_config_path = REPO_BASE_DIR / "repoconfig.yaml"
    except FileNotFoundError:
        print("Please put the repoconfig.yaml in the main directory structure")
        sys.exit(1)
    return load_url_dict(repo_config_path)


def load_url_dict(repo_config_path):
    yamlload = load_yaml(repo_config_path)

    try:
        urldict = yamlload["Projectproperties"]
        return urldict

    except KeyError:
        print("Please use the right format for the config file")
        sys.exit(1)



import os
import shutil
import time
from pathlib import Path
from tempfile import TemporaryDirectory

from cognite.client import CogniteClient
from cognite.experimental import CogniteClient as ExperimentalClient

from cognite.airworkflow.constants import PROJECTS_TO_API_KEYS

os.environ["COGNITE_MAX_RETRIES"] = "5"
os.environ["COGNITE_TIMEOUT"] = "120"


def data_set_id(client: CogniteClient, name: str) -> int:
    data_sets = client.data_sets.list(limit=-1).dump()
    for ds in data_sets:
        if "name" in ds:
            if ("archived" not in ds["metadata"] or ds["metadata"]["archived"] != "true") and ds["name"] == name:
                return ds["id"]
    raise LookupError(f"Not able to find dataset named {name}")


def zip_and_upload(client: CogniteClient, code_path: Path, file_name: str, data_set_name: str) -> int:
    print(f"Uploading code from {code_path} ... ", flush=True)
    with TemporaryDirectory() as tmpdir:
        zip_path = Path(tmpdir) / "function"
        shutil.make_archive(str(zip_path), "zip", str(code_path))
        id = data_set_id(client, data_set_name)
        file = client.files.upload(
            f"{zip_path}.zip",
            name=f"{file_name}.zip",
            external_id=f"{file_name}.zip",
            overwrite=True,
            data_set_id=id,
        )
        counter = 0
        while client.files.retrieve(file.id) is None:
            time.sleep(0.5)
            counter += 1
            if counter > 5:
                break
        print(f"Upload complete. \n{file}", flush=True)
        return file.id


def does_function_exist(client: ExperimentalClient, function_name: str) -> bool:
    return bool(client.functions.retrieve(external_id=function_name))


def await_function_deployment(client: ExperimentalClient, function_name: str, max_wait: int) -> bool:
    t_end = time.time() + max_wait
    while time.time() < t_end:
        function = client.functions.retrieve(external_id=function_name)
        print(f"Function status {function.status}")
        if function.status == "Ready":
            return True
        if function.status == "Failed":
            return False
        time.sleep(5.0)

    return False


def experimental_client(project: str, api_key: str = "", base_url: str = None) -> ExperimentalClient:
    return ExperimentalClient(
        api_key=api_key if api_key else PROJECTS_TO_API_KEYS[project].get_client_api_key(),
        client_name="AIR Functions Client",
        project=project,
        base_url=base_url if base_url else PROJECTS_TO_API_KEYS[project].base_url,
    )
